@@ -66,6 +66,7 @@ gate_aliases = [
 class AbstractCircuit:
     _nqubits: int
     _qir: List[Dict[str, Any]]
+    _extra_qir: List[Dict[str, Any]]
     inputs: Tensor
     circuit_param: Dict[str, Any]
     is_mps: bool
@@ -453,7 +454,9 @@ class AbstractCircuit:
                 name = g1
                 break
         if name not in sgates + vgates + mpogates:
-            logger.warning("gate name not in the common gate set that tc supported")
+            logger.warning(
+                "gate name %s not in the common gate set that tc supported" % name
+            )
         return name
 
     def gate_count(self, gate_list: Optional[Sequence[str]] = None) -> int:
@@ -498,16 +501,71 @@ class AbstractCircuit:
             summary[d["name"]] = summary.get(d["name"], 0) + 1
         return summary
 
-    def to_qiskit(self) -> Any:
+    def measure_instruction(self, index: int) -> None:
+        """
+        add a measurement instruction flag, no effect on numerical simulation
+
+        :param index: the corresponding qubit
+        :type index: int
+        """
+        l = len(self._qir)
+        d = {
+            "index": index,
+            "name": "measure",
+            "gatef": "measure",
+            "instruction": True,
+            "pos": l,
+        }
+        self._extra_qir.append(d)
+
+    def reset_instruction(self, index: int) -> None:
+        """
+        add a reset instruction flag, no effect on numerical simulation
+
+        :param index: the corresponding qubit
+        :type index: int
+        """
+        l = len(self._qir)
+        d = {
+            "index": index,
+            "name": "reset",
+            "gatef": "reset",
+            "instruction": True,
+            "pos": l,
+        }
+        self._extra_qir.append(d)
+
+    def barrier_instruction(self, *index: List[int]) -> None:
+        """
+        add a barrier instruction flag, no effect on numerical simulation
+
+        :param index: the corresponding qubits
+        :type index: List[int]
+        """
+        l = len(self._qir)
+        d = {
+            "index": index,
+            "name": "barrier",
+            "gatef": "barrier",
+            "instruction": True,
+            "pos": l,
+        }
+        self._extra_qir.append(d)
+
+    def to_qiskit(self, enable_instruction: bool = False) -> Any:
         """
         Translate ``tc.Circuit`` to a qiskit QuantumCircuit object.
 
+        :param enable_instruction: whether also export measurement and reset instructions
+        :type enable_instruction: bool, defaults to False
         :return: A qiskit object of this circuit.
         """
         from .translation import qir2qiskit
 
         qir = self.to_qir()
-        return qir2qiskit(qir, n=self._nqubits)
+        if enable_instruction is False:
+            return qir2qiskit(qir, n=self._nqubits)
+        return qir2qiskit(qir, n=self._nqubits, extra_qir=self._extra_qir)
 
     def to_openqasm(self, **kws: Any) -> str:
         """
@@ -518,25 +576,43 @@ class AbstractCircuit:
         :return: circuit representation in openqasm format
         :rtype: str
         """
-        return self.to_qiskit().qasm(**kws)  # type: ignore
+        return self.to_qiskit(enable_instruction=True).qasm(**kws)  # type: ignore
 
     @classmethod
     def from_openqasm(
-        cls, qasmstr: str, circuit_params: Optional[Dict[str, Any]] = None
+        cls,
+        qasmstr: str,
+        circuit_params: Optional[Dict[str, Any]] = None,
+        keep_measure_order: bool = False,
     ) -> "AbstractCircuit":
         from qiskit.circuit import QuantumCircuit
 
-        qiskit_circ = QuantumCircuit.from_qasm_str(qasmstr)
+        if keep_measure_order is True:
+            from .translation import qiskit_from_qasm_str_ordered_measure
+
+            qiskit_circ = qiskit_from_qasm_str_ordered_measure(qasmstr)
+        else:
+            qiskit_circ = QuantumCircuit.from_qasm_str(qasmstr)
         c = cls.from_qiskit(qiskit_circ, circuit_params=circuit_params)
         return c
 
     @classmethod
     def from_openqasm_file(
-        cls, file: str, circuit_params: Optional[Dict[str, Any]] = None
+        cls,
+        file: str,
+        circuit_params: Optional[Dict[str, Any]] = None,
+        keep_measure_order: bool = False,
     ) -> "AbstractCircuit":
         from qiskit.circuit import QuantumCircuit
 
-        qiskit_circ = QuantumCircuit.from_qasm_file(file)
+        if keep_measure_order is True:
+            from .translation import qiskit_from_qasm_str_ordered_measure
+
+            with open(file) as f:
+                qasmstr = f.read()
+            qiskit_circ = qiskit_from_qasm_str_ordered_measure(qasmstr)
+        else:
+            qiskit_circ = QuantumCircuit.from_qasm_file(file)
         c = cls.from_qiskit(qiskit_circ, circuit_params=circuit_params)
         return c
 
@@ -559,7 +635,7 @@ class AbstractCircuit:
         q_2: ┤ X ├─────
              └───┘
         """
-        return self.to_qiskit().draw(**kws)
+        return self.to_qiskit(enable_instruction=True).draw(**kws)
 
     @classmethod
     def from_qiskit(
@@ -831,7 +907,9 @@ class AbstractCircuit:
         self.__dict__.update(newc.__dict__)
         return self
 
-    def append(self, c: "AbstractCircuit") -> "AbstractCircuit":
+    def append(
+        self, c: "AbstractCircuit", indices: Optional[List[int]] = None
+    ) -> "AbstractCircuit":
         """
         append circuit ``c`` before
 
@@ -853,11 +931,22 @@ class AbstractCircuit:
 
         :param c: The other circuit to be appended
         :type c: BaseCircuit
+        :param indices: the qubit indices to which ``c`` is appended on.
+            Defaults to None, which means plain concatenation.
+        :type indices: Optional[List[int]], optional
         :return: The composed circuit
         :rtype: BaseCircuit
         """
         qir1 = self.to_qir()
         qir2 = c.to_qir()
+        if indices is not None:
+            qir2_old = qir2
+            qir2 = []
+            for d in qir2_old:
+                # avoid modifying the original circuit
+                d = d.copy()
+                d["index"] = [indices[i] for i in d["index"]]
+                qir2.append(d)
         newc = type(self).from_qir(qir1 + qir2, self.circuit_param)
         self.__dict__.update(newc.__dict__)
         return self
