@@ -351,10 +351,10 @@ class PyTorchBackend(pytorch_backend.PyTorchBackend, ExtendedBackend):  # type:
         return torchlib.unique(a, return_counts=True)  # type: ignore
 
     def sigmoid(self, a: Tensor) -> Tensor:
-        return torchlib.nn.Sigmoid(a)
+        return torchlib.sigmoid(a)
 
     def relu(self, a: Tensor) -> Tensor:
-        return torchlib.nn.ReLU(a)
+        return torchlib.relu(a)
 
     def softmax(self, a: Sequence[Tensor], axis: Optional[int] = None) -> Tensor:
         return torchlib.nn.Softmax(a, dim=axis)
