@@ -368,6 +368,11 @@ class TensorFlowBackend(tensorflow_backend.TensorFlowBackend, ExtendedBackend):
     ) -> Tensor:
         return tf.math.reduce_mean(a, axis=axis, keepdims=keepdims)
 
+    def std(
+        self, a: Tensor, axis: Optional[Sequence[int]] = None, keepdims: bool = False
+    ) -> Tensor:
+        return tf.math.reduce_std(a, axis=axis, keepdims=keepdims)
+
     def sigmoid(self, a: Tensor) -> Tensor:
         return tf.nn.sigmoid(a)
 
@@ -687,6 +692,11 @@ class TensorFlowBackend(tensorflow_backend.TensorFlowBackend, ExtendedBackend):
             t.watch(inputs)
             y = f(*inputs)
         g = t.gradient(y, inputs, v)
+        g = list(g)
+        for i, gi in enumerate(g):
+            if gi is None:
+                g[i] = tf.zeros_like(inputs[i])
+        g = tuple(g)
         if one_input:
             g = g[0]
         return y, g
