@@ -503,6 +503,21 @@ class BaseCircuit(AbstractCircuit):
             no.extend(msconj)
         return contractor(no).tensor
 
+    def probability(self) -> Tensor:
+        """
+        get the 2^n length probability vector over computational basis
+
+        :return: probability vector
+        :rtype: Tensor
+        """
+        s = self.state()  # type: ignore
+        if self.is_dm is False:
+            p = backend.abs(s) ** 2
+
+        else:
+            p = backend.abs(backend.diagonal(s))
+        return p
+
     @partial(arg_alias, alias_dict={"format": ["format_"]})
     def sample(
         self,
@@ -519,7 +534,7 @@ class BaseCircuit(AbstractCircuit):
         :param batch: number of samples, defaults to None
         :type batch: Optional[int], optional
         :param allow_state: if true, we sample from the final state
-            if memory allsows, True is prefered, defaults to False
+            if memory allows, True is preferred, defaults to False
         :type allow_state: bool, optional
         :param readout_error: readout_error, defaults to None
         :type readout_error: Optional[Sequence[Any]]. Tensor, List, Tuple
@@ -531,8 +546,8 @@ class BaseCircuit(AbstractCircuit):
         :param status: external randomness given by tensor uniformly from [0, 1],
             if set, can overwrite random_generator
         :type status: Optional[Tensor]
-        :return: List (if batch) of tuple (binary configuration tensor and correponding probability)
-            if the format is None, and consitent with format when given
+        :return: List (if batch) of tuple (binary configuration tensor and corresponding probability)
+            if the format is None, and consistent with format when given
         :rtype: Any
         """
         # allow_state = False is compatibility issue
@@ -569,20 +584,11 @@ class BaseCircuit(AbstractCircuit):
                 nbatch = 1
             else:
                 nbatch = batch
-            s = self.state()  # type: ignore
-            if self.is_dm is False:
-                p = backend.abs(s) ** 2
+            p = self.probability()
 
-                # readout error
-                if readout_error is not None:
-                    p = self.readouterror_bs(readout_error, p)
-
-            else:
-                p = backend.abs(backend.diagonal(s))
-
-                # readout error
-                if readout_error is not None:
-                    p = self.readouterror_bs(readout_error, p)
+            # readout error
+            if readout_error is not None:
+                p = self.readouterror_bs(readout_error, p)
             ch = backend.probability_sample(nbatch, p, status, random_generator)
             # if random_generator is None:
             #     ch = backend.implicit_randc(a=a_range, shape=[nbatch], p=p)
@@ -663,7 +669,7 @@ class BaseCircuit(AbstractCircuit):
         :param status: external randomness given by tensor uniformly from [0, 1],
             if set, can overwrite random_generator
         :type status: Optional[Tensor]
-        :param readout_error: readout_error, defaults to None
+        :param readout_error: readout_error, defaults to None. Overrided if noise_conf is provided.
         :type readout_error: Optional[Sequence[Any]]. Tensor, List, Tuple
         :param noise_conf: Noise Configuration, defaults to None
         :type noise_conf: Optional[NoiseConf], optional
@@ -811,7 +817,7 @@ class BaseCircuit(AbstractCircuit):
         else:  # TODO(@refraction-ray) replace several start as inputs
             raise NotImplementedError("not support replace with no inputs")
 
-    def cond_measurement(self, index: int) -> Tensor:
+    def cond_measurement(self, index: int, status: Optional[float] = None) -> Tensor:
         """
         Measurement on z basis at ``index`` qubit based on quantum amplitude
         (not post-selection). The highlight is that this method can return the
@@ -842,6 +848,7 @@ class BaseCircuit(AbstractCircuit):
         return self.general_kraus(  # type: ignore
             [np.array([[1.0, 0], [0, 0]]), np.array([[0, 0], [0, 1]])],
             index,
+            status=status,
             name="measure",
         )
 
