@@ -109,6 +109,7 @@ class Circuit(BaseCircuit):
         # self._qcode = ""  # deprecated
         # self._qcode += str(self._nqubits) + "\n"
         self._qir: List[Dict[str, Any]] = []
+        self._extra_qir: List[Dict[str, Any]] = []
 
     def replace_mps_inputs(self, mps_inputs: QuOperator) -> None:
         """
@@ -408,9 +409,14 @@ class Circuit(BaseCircuit):
         l = int(prob.shape[0])  # type: ignore
 
         def step_function(x: Tensor) -> Tensor:
-            r = backend.sum(
-                backend.stack([backend.sign(x - prob_cumsum[i]) for i in range(l - 1)])
-            )
+            if l == 1:
+                r = backend.convert_to_tensor(0.0)
+            else:
+                r = backend.sum(
+                    backend.stack(
+                        [backend.sign(x - prob_cumsum[i]) for i in range(l - 1)]
+                    )
+                )
             r = backend.cast(r / 2.0 + (l - 1) / 2.0, dtype="int32")
             # [0: kraus[0], 1: kraus[1]...]
             return r
