@@ -413,6 +413,16 @@ def test_expectation_ps():
     np.testing.assert_allclose(r, 1, atol=1e-5)
 
 
+def test_probability():
+    for c_cls in [tc.Circuit, tc.DMCircuit]:
+        c = c_cls(2)
+        c.h(0)
+        c.h(1)
+        np.testing.assert_allclose(
+            c.probability(), np.array([1, 1, 1, 1]) / 4, atol=1e-5
+        )
+
+
 @pytest.mark.parametrize("backend", [lf("tfb"), lf("jaxb")])
 def test_dqas_type_circuit(backend):
     eye = tc.gates.i().tensor
@@ -785,6 +795,12 @@ def test_append_circuit(backend):
     c.prepend(c1)
     np.testing.assert_allclose(c.expectation_ps(z=[1]), -1.0)
 
+    c = tc.Circuit(2)
+    c1 = tc.Circuit(1)
+    c1.x(0)
+    c.append(c1, [1])
+    np.testing.assert_allclose(c.state(), [0, 1, 0, 0])
+
 
 @pytest.mark.parametrize("backend", [lf("npb"), lf("tfb"), lf("jaxb")])
 def test_apply_mpo_gate(backend):
@@ -964,6 +980,11 @@ def test_qiskit2tc():
     qisc.swap(0, 1)
     qisc.iswap(0, 1)
     qisc.toffoli(0, 1, 2)
+    # test Instructions
+    qisc2 = QuantumCircuit(1)
+    qisc2.h(0)
+    qisc.compose(qisc2, qubits=[1], inplace=True, wrap=True)
+    qisc.barrier(0, 1, 2)
     qisc.s(1)
     qisc.t(1)
     qisc.sdg(2)
@@ -1002,6 +1023,10 @@ def test_qiskit2tc():
     qis_unitary = np.reshape(qis_unitary, [2**n, 2**n])
     p_mat = perm_matrix(n)
     np.testing.assert_allclose(p_mat @ tc_unitary @ p_mat, qis_unitary, atol=1e-5)
+    qisc_from_tc = c.to_qiskit(enable_instruction=True)
+    qis_unitary2 = qi.Operator(qisc_from_tc)
+    qis_unitary2 = np.reshape(qis_unitary2, [2**n, 2**n])
+    np.testing.assert_allclose(qis_unitary2, qis_unitary, atol=1e-5)
 
 
 @pytest.mark.parametrize("backend", [lf("npb"), lf("tfb"), lf("jaxb")])
@@ -1222,6 +1247,7 @@ def test_to_openqasm():
     c.rzz(0, 1, theta=-1.0)
     c.ccx(1, 2, 0)
     c.u(2, theta=0.5, lbd=1.3)
+    c.measure_instruction(1)
     print(c.to_openqasm(formatted=True))
     s = c.to_openqasm()
     c1 = tc.Circuit.from_openqasm(s)
@@ -1230,3 +1256,17 @@ def test_to_openqasm():
     c.to_openqasm(filename="test.qasm")
     c2 = tc.Circuit.from_openqasm_file("test.qasm")
     np.testing.assert_allclose(c.state(), c2.state())
+
+
+def test_from_qasm_keep_measure_order():
+    qasm_str = """OPENQASM 2.0;
+include "qelib1.inc";
+qreg q[2];
+creg c[2];
+h q[0];
+measure q[1] -> c[1];
+measure q[0] -> c[0];"""
+    c = tc.Circuit.from_openqasm(qasm_str)
+    c.to_openqasm().split("\n")[-2][-3] == "1"
+    c = tc.Circuit.from_openqasm(qasm_str, keep_measure_order=True)
+    c.to_openqasm().split("\n")[-2][-3] == "0"
