@@ -5,7 +5,6 @@ from typing import Tuple, Any
 import sys
 import os
 import numpy as np
-import scipy
 import pytest
 from pytest_lazyfixture import lazy_fixture as lf
 
@@ -18,56 +17,46 @@ sys.path.insert(0, modulepath)
 import tensorcircuit as tc
 
 
-N = 16
-D = 100
+N = 8
+D = 6
 split = tc.cons.split_rules(max_singular_values=D)
 type_test_circuits = Tuple[
     tc.Circuit, Tensor, tc.MPSCircuit, Tensor, tc.MPSCircuit, Tensor
 ]
 
 
-def reproducible_unitary(n):
-    A = np.arange(n**2).reshape((n, n))
-    A = A + np.sin(A) * 1j
-    A = A - A.conj().T
-    return scipy.linalg.expm(A).astype(tc.dtypestr)
+def reproducible_unitary(n, param):
+    exp2n = 2**n
+    A = tc.backend.cast(
+        tc.backend.reshape(tc.backend.arange(exp2n**2), (exp2n, exp2n)), tc.dtypestr
+    )
+    A = A + tc.backend.sin(A) * param * 1j
+    A = A - tc.backend.conj(tc.backend.transpose(A))
+    return tc.backend.reshape(tc.backend.expm(A), (2,) * n * 2)
 
 
-def get_test_circuits(full) -> type_test_circuits:
-    O1 = tc.gates.any(reproducible_unitary(2).reshape((2, 2)))
-    O2 = tc.gates.any(reproducible_unitary(4).reshape((2, 2, 2, 2)))
-    O3 = tc.gates.any(reproducible_unitary(8).reshape((2, 2, 2, 2, 2, 2)))
+def simulate(c, check=True, params=None):
+    if params is None:
+        params = tc.backend.ones((3,), dtype=tc.dtypestr)
+    O1 = tc.gates.any(reproducible_unitary(1, params[0]))
+    O2 = tc.gates.any(reproducible_unitary(2, params[1]))
+    O3 = tc.gates.any(reproducible_unitary(3, params[2]))
 
     # Construct a complicated circuit by Circuit and MPSCircuit and compare
 
-    if full:
+    c.H(0)
+    # create as much correlation as possible
+    for i in range(0, N - 1, 2):
+        c.apply(O2.copy(), i, i + 1)
+        c.apply(O1.copy(), i)
+    c.apply(O3.copy(), int(N * 0.1), int(N * 0.5), int(N * 0.9))
+    c.apply(O2.copy(), 1, N - 2)
+    if check and isinstance(c, tc.MPSCircuit):
+        np.testing.assert_allclose(np.abs(c._mps.check_canonical()), 0, atol=1e-12)
+    c.cz(2, 3)
 
-        def rangei(j, N):
-            return range(0, N - 1)
-
-    else:
-
-        def rangei(j, N):
-            return range(j, N - 1 - j)
-
-    def simulate(c):
-        c.H(0)
-        # create as much correlation as possible
-        for j in range(N // 2):
-            for i in rangei(j, N):
-                c.apply(O2.copy(), i, i + 1)
-                c.apply(O1.copy(), i)
-        # test non-adjacent double gates
-        c.apply(O2.copy(), N // 2 - 1, N // 2 + 1)
-        c.apply(O3.copy(), int(N * 0.2), int(N * 0.4), int(N * 0.6))
-        if isinstance(c, tc.MPSCircuit):
-            np.testing.assert_allclose(np.abs(c._mps.check_canonical()), 0, atol=1e-12)
-        c.apply(O2.copy(), N // 2 - 2, N // 2 + 2)
-        c.apply(O3.copy(), int(N * 0.4), int(N * 0.6), int(N * 0.8))
-        if isinstance(c, tc.MPSCircuit):
-            np.testing.assert_allclose(np.abs(c._mps.check_canonical()), 0, atol=1e-12)
-        c.cz(2, 3)
 
+def get_test_circuits() -> type_test_circuits:
     c = tc.Circuit(N)
     simulate(c)
     w_c = c.wavefunction()
@@ -133,10 +122,13 @@ def do_test_truncation(
         np.abs(tc.backend.numpy(w_mps).conj().dot(tc.backend.numpy(w_c))) ** 2
     )
     estimated_fedility = tc.backend.numpy(mps._fidelity)
-    print(real_fedility)
-    print(estimated_fedility)
-    np.testing.assert_allclose(real_fedility, real_fedility_ref, atol=1e-8)
-    np.testing.assert_allclose(estimated_fedility, estimated_fedility_ref, atol=1e-8)
+    print(real_fedility, estimated_fedility)
+    if real_fedility_ref is not None:
+        np.testing.assert_allclose(real_fedility, real_fedility_ref, atol=1e-5)
+    if estimated_fedility_ref is not None:
+        np.testing.assert_allclose(
+            estimated_fedility, estimated_fedility_ref, atol=1e-5
+        )
 
 
 def do_test_amplitude(test_circuits: type_test_circuits):
@@ -165,14 +157,14 @@ def do_test_expectation(test_circuits: type_test_circuits):
         w_mps_exact,
     ) = test_circuits
 
-    single_gate = (tc.gates.z(), [11])
+    single_gate = (tc.gates.z(), [3])
     tensor = (np.sin(np.arange(16)) + np.cos(np.arange(16)) * 1j).reshape((2, 2, 2, 2))
     double_gate_nonunitary = (
         tc.gates.Gate(tc.backend.convert_to_tensor(tensor)),
         [2, 6],
     )
     double_gate = (tc.gates.cnot(), [2, 6])
-    triple_gate = (tc.gates.toffoli(), [1, 5, 9])
+    triple_gate = (tc.gates.toffoli(), [7, 1, 5])
     gates = [single_gate, double_gate_nonunitary, triple_gate]
 
     exp_mps = mps_exact.expectation(*gates)
@@ -181,8 +173,8 @@ def do_test_expectation(test_circuits: type_test_circuits):
 
     # ps
     x = [0, 2]
-    y = [1, 3, 5]
-    z = [6, 8, 10]
+    y = [5, 3, 1]
+    z = [6, 4]
     exp_mps = mps_exact.expectation_ps(x=x, y=y, z=z)
     exp_c = c.expectation_ps(x=x, y=y, z=z)
     np.testing.assert_allclose(exp_mps, exp_c, atol=1e-7)
@@ -216,8 +208,8 @@ def do_test_fromwavefunction(external_wavefunction, relative_err_ref):
     s = np.linalg.svd(w_external.reshape((2 ** (N // 2), 2 ** (N // 2))))[1]
     theoretical_upper_limit = np.sum(s[0:D] ** 2)
     relative_err = np.log((1 - real_fedility) / (1 - theoretical_upper_limit))
-    print(relative_err)
-    np.testing.assert_allclose(relative_err, 0.11, atol=1e-2)
+    if relative_err_ref is not None:
+        np.testing.assert_allclose(relative_err, relative_err_ref, atol=1e-4)
 
 
 def do_test_proj(test_circuits: type_test_circuits, external_wavefunction):
@@ -273,22 +265,17 @@ def do_test_measure(test_circuits: type_test_circuits):
         mps_exact,
         w_mps_exact,
     ) = test_circuits
-    index = [6, 5, 2, 9]
-    status = tc.backend.convert_to_tensor([0.1, 0.3, 0.5, 0.7])
+    index = [6, 5, 2, 1]
+    status = tc.backend.convert_to_tensor([0.1, 0.3, 0.7, 0.9])
     result_c = c.measure(*index, with_prob=True, status=status)
-    result_mps = mps.measure(*index, with_prob=True, status=status)
     result_mps_exact = mps_exact.measure(*index, with_prob=True, status=status)
-    np.testing.assert_allclose(result_mps[0], result_c[0], atol=1e-8)
     np.testing.assert_allclose(result_mps_exact[0], result_c[0], atol=1e-8)
-    np.testing.assert_allclose(result_mps[1], result_c[1], atol=1e-4)
     np.testing.assert_allclose(result_mps_exact[1], result_c[1], atol=1e-8)
 
 
 def test_MPO_conversion(highp, tfb):
-    O3 = tc.backend.convert_to_tensor(
-        reproducible_unitary(8).reshape((2, 2, 2, 2, 2, 2))
-    )
-    I = tc.backend.convert_to_tensor(np.eye(2).astype("complex128"))
+    O3 = reproducible_unitary(3, 1.0)
+    I = tc.backend.eye(2, dtype=tc.dtypestr)
     gate = tc.gates.Gate(O3)
 
     MPO3, _ = tc.MPSCircuit.gate_to_MPO(gate, 2, 3, 4)
@@ -308,25 +295,47 @@ def test_MPO_conversion(highp, tfb):
 @pytest.mark.parametrize(
     "backend, dtype", [(lf("tfb"), lf("highp")), (lf("jaxb"), lf("highp"))]
 )
-def test_circuits_1(backend, dtype):
-    import time
-
-    begin = time.time()
-    circuits = get_test_circuits(False)
-    print("time", time.time() - begin)
+def test_circuits(backend, dtype):
+    circuits = get_test_circuits()
     do_test_canonical(circuits)
     do_test_wavefunction(circuits)
-    do_test_truncation(circuits, 0.9987293417932497, 0.999440840610151)
+    do_test_truncation(circuits, 0.902663090851, 0.910305380327)
     do_test_amplitude(circuits)
     do_test_expectation(circuits)
     external = external_wavefunction()
-    do_test_fromwavefunction(external, 0.1185)
+    do_test_fromwavefunction(external, 0.276089)
     do_test_proj(circuits, external)
     do_test_tensor_input(circuits)
     do_test_measure(circuits)
-    print("time", time.time() - begin)
 
 
-def test_circuits_2(highp):
-    circuits = get_test_circuits(True)
-    do_test_truncation(circuits, 0.9401410770899974, 0.9654331011546374)
+@pytest.mark.parametrize("backend, dtype", [(lf("tfb"), lf("highp"))])
+def test_circuits_jit(backend, dtype):
+    def expec(params):
+        mps = tc.MPSCircuit(N, split=split)
+        simulate(mps, check=False, params=params)
+        x = [0, 2]
+        y = [5, 3, 1]
+        z = [6, 4]
+        exp = mps.expectation_ps(x=x, y=y, z=z)
+        return tc.backend.real(exp)
+
+    params = tc.backend.ones((3,), dtype=tc.dtypestr)
+    expec_vg = tc.backend.value_and_grad(expec)
+    expec_vg_jit = tc.backend.jit(expec_vg)
+    exp = expec(params)
+    exp_jit, exp_grad_jit = expec_vg_jit(params)
+    dir = tc.backend.convert_to_tensor(np.array([1.0, 2.0, 3.0], dtype=tc.dtypestr))
+    epsilon = 1e-6
+    exp_p = expec(params + dir * epsilon)
+    exp_m = expec(params - dir * epsilon)
+    exp_grad_dir_numerical = (exp_p - exp_m) / (epsilon * 2)
+    exp_grad_dir_jit = tc.backend.real(tc.backend.sum(exp_grad_jit * dir))
+    np.testing.assert_allclose(
+        tc.backend.numpy(exp), tc.backend.numpy(exp_jit), atol=1e-10
+    )
+    np.testing.assert_allclose(
+        tc.backend.numpy(exp_grad_dir_numerical),
+        tc.backend.numpy(exp_grad_dir_jit),
+        atol=1e-6,
+    )
